extern crate clap;

mod lifx;
mod light;
mod mqtt;

use crate::mqtt::Mqtt;
use clap::App;
use clap::Arg;
#[macro_use]
extern crate serde_derive;

pub const MQTT_ID: &str = "lifx-mqtt-bridge";

fn main() {
    let matches = App::new("lifx-mqtt-bridge")
        .version("0.1")
        .about("Lifx Mqtt Bridge")
        .author("Buntpfotenkätzchen")
        .arg(
            Arg::with_name("host")
                .short("h")
                .long("host")
                .required(true)
                .takes_value(true),
        )
        .arg(
            Arg::with_name("port")
                .short("p")
                .long("port")
                .takes_value(true)
                .default_value("1883"),
        )
        .arg(
            Arg::with_name("lifx_secret")
                .short("s")
                .long("lifx_secret")
                .required(true)
                .takes_value(true),
        )
        .get_matches();

    let host = matches.value_of("host").unwrap();
    let port: u16 = matches.value_of("port").unwrap().parse().unwrap();
    let lifx_secret = matches.value_of("lifx_secret").unwrap();
    println!("Connecting to {}:{}", host, port);

    let mut mqtt = match Mqtt::connect(host, port) {
        Ok(mqtt) => mqtt,
        Err(err) => panic!("Error connecting: {}", err),
    };

    let lifx_client = lifx::Lifx::new(lifx_secret);
    let lights = lifx_client.find_lights();
    println!("lights: {:#?}", lights);

    for light in lights {
        mqtt.add_light(&light.id, &light.label);
    }

    loop {
        match mqtt.notifications.recv() {
            Ok(notification) => {
                println!("MQTT notification received: {:#?}", notification);
            }
            Err(recv_error) => {
                println!("MQTT channel closed: {}", recv_error);
                return;
            }
        }
    }
}
